/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef ___ALSARECORDER_HPP___
#define ___ALSARECORDER_HPP___

#include "Recorder.hpp"

#include <alsa/asoundlib.h>

class ALSARecorder: public Recorder {
private:
  static std::unique_ptr<ALSARecorder> instance;
  snd_pcm_t* p_pcm;
  std::shared_ptr<std::thread> recorderThread;
  std::vector<short> read_buf;
  int read_buf_size;

public:
  ALSARecorder() :
      p_pcm(0), read_buf_size(128) {
    //cerr << "ALSARecorder\n";
  }

  static Recorder* GetInstance() {
    return new ALSARecorder;
  }

  void operator()();
  virtual int Initialize(const std::string& device_name, int channel_count, int sampling_rate, size_t buf_size);

  // start recording
  virtual void Start() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    state = RECORDING;
  }

  // pause in recording
  virtual void Pause() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    state = PAUSE;
  }

  // stop and finish recording
  virtual void Stop() {
    std::unique_lock<std::mutex> lk(mutex_buffer);
    state = STOP;
  }

  virtual void Close() {
  }
};

#endif
